/**
 *  Jonathan Sun
 */


import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

public class CSVPlayer {

    final static int MEASURE_DUR = 2000; // Default measure duration, milliseconds.
    final static int PITCH = 0;
    final static int DUR = 1;

    public static void printUsage() {
        System.err.println(
            "Usage: java CSVPlayer </my/path/to/file.csv>"
        );
        System.exit(1);
    }

    public static int[] calcDurs(int[] dur1s, int[] dur2s) {
        int[] durs = new int[dur1s.length];
        for (int i = 0; i < dur1s.length; i++)
            durs[i] = (int)((double)MEASURE_DUR * ((double)dur1s[i] / (double)dur2s[i]));
        return durs;
    }

    /**
     * Converts a string of comma separated integers to an array of integers.
     * @param csv comma separated integers
     * @return integer array
     */
    public static int[] scanInts(String csv) {
        String[] nums = csv.split(",");
        int[] ints = new int[nums.length];
        for (int i = 0; i < nums.length; i++)
            ints[i] = Integer.parseInt(nums[i]);
        return ints;
    }

    public static ArrayList<int[][]> buildScore(Scanner input) {
        ArrayList<int[][]> score = new ArrayList<int[][]>();

        /* no error handling... */
        while (input.hasNextLine()) {
            int[][] voice = new int[2][];
            int[] pitches = scanInts(input.nextLine());
            int[] dur1s = scanInts(input.nextLine());
            int[] dur2s = scanInts(input.nextLine());
            int[] durs = calcDurs(dur1s, dur2s);

            voice[PITCH] = pitches;
            voice[DUR] = durs;

            score.add(voice);
        }
        return score;
    }

    public static void play(ArrayList<int[][]> score) {
        ArrayList<NotesPlayer> voices = new ArrayList<NotesPlayer>();
        for (int[][] voice : score)
            voices.add(new NotesPlayer(voice));
        for (NotesPlayer notes : voices)
            (new Thread(notes)).start();
    }

    public static void main(String[] args) throws FileNotFoundException {
        if (args.length != 1) printUsage();

        Scanner input = new Scanner(new File(args[0]));
        ArrayList<int[][]> score = buildScore(input);
        input.close();

        play(score);

        System.out.println(
            "Thank you for your patronage. Have a nice day."
        );
    }
}
