/**
 * Jonathan Sun
 */

import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Synthesizer;
import java.lang.Runnable;
import java.lang.Thread;

public class NotesPlayer implements Runnable {

    final static int VOLUME = 100;
    final static int PITCH = 0;
    final static int DUR = 1;
    final static int MEASURE_DUR = 2000; // Default measure duration

    public NotesPlayer(int[][] notes) {
        this.notes = notes;
    }

    @Override
    public void run() {
        try {
            playNotes();
        } catch (MidiUnavailableException e) {
            threadMessage("Midi Unavailable");
        } catch (InterruptedException e) {
            threadMessage("Thread interrupted.");
        }
    }

    private void playNotes() throws MidiUnavailableException, InterruptedException {
        Synthesizer synth = MidiSystem.getSynthesizer();
        MidiChannel[] channels = synth.getChannels();
        synth.open();
        for (int i = 0; i < notes[0].length; i++) {
            System.out.println("Playing: " + notes[PITCH][i]);
            if (notes[PITCH][i] == -1) {
                Thread.sleep(notes[DUR][i]);
            } else {
                channels[0].noteOn(notes[PITCH][i], VOLUME);
                Thread.sleep(notes[DUR][i]);
                channels[0].noteOff(notes[PITCH][i], VOLUME);
            }
        }
        synth.close();
    }
    
    private void threadMessage(String message) {
        String threadName = Thread.currentThread().getName();
        System.out.format("%s: %s%n", threadName, message);
    }

    private int[][] notes;
}
