%{ 
(*
Parser for CMajor
By PLT Sandwich
Andrew OReilly, Stephanie Huang
*)
open Ast 
%}

%token PLUS MINUS TIMES DIVIDE LAYER REPEAT ARRCAT ASSIGN EQ NEQ GT GTE LT LTE EOF
%token LPAREN RPAREN LCBRACE RCBRACE LSBRACE RSBRACE COMMA SEMI RETURN IF ELSE FOR
%token PITCH_LIT
%token <int> INT_LIT
%token <int> PITCH_LETTER
%token <int> PITCH_SIGN
%token <string> ID
%token <string> TYPENAME

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left ARRCAT
%left REPEAT
%left EQ NEQ
%left LT GT LTE GTE
%left PLUS MINUS
%left TIMES DIVIDE
%left LAYER

%start program
%type < Ast.program> program
%type < Ast.expr> expr
%type < Ast.vdecl> vdecl
%%

program:
  decls EOF { $1 }
  
decls:
    { [], [] }
  | decls vdecl SEMI { ($2 :: fst $1), snd $1 }
  | decls fdecl { fst $1, ($2 :: snd $1) }

vdecl:
    TYPENAME ID { $1, $2                      (* Variable declaration *) }
  | TYPENAME LSBRACE RSBRACE ID { $1^"[]", $4 (* Array declaration *) } 

fdecl:
  TYPENAME ID LPAREN formals_opt RPAREN LCBRACE stmt_list RCBRACE
  {
    {
      ftype = $1;
      fname = $2;
      formals = $4;
      locals = [];
      body = List.rev $7; 
    }
  }

formals_opt:
    { [] }
  | formal_list { List.rev $1 }
  
formal_list:
    vdecl { [$1] }
  | formal_list COMMA vdecl { $3 :: $1 }

stmt_list:
    { [] }
  | stmt_list stmt { $2 :: $1 }
  
stmt:
    expr SEMI                               { Expr($1) }
  | vdecl SEMI                              { Vdecl(fst $1, snd $1) }
  | vdecl ASSIGN expr SEMI                  { VdeclAsn(Vdecl(fst $1, snd $1), Asn(snd $1, $3)) }
  | RETURN expr SEMI                        { Return($2) }
  | LCBRACE stmt_list RCBRACE               { Block(List.rev $2) }
  | IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
  | IF LPAREN expr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LPAREN expr_opt SEMI expr_opt SEMI expr_opt RPAREN stmt 
                                            { For($3, $5, $7, $9) }

expr_opt:
         { Noexpr }
  | expr { $1 }
                                            
expr:
  ID ASSIGN expr                  { Asn($1, $3) }
| LSBRACE actuals_opt RSBRACE     { Arr($2) }
| ID LPAREN actuals_opt RPAREN    { Call($1, $3) }
| expr PLUS   expr                { Binop($1, Add, $3)    (* Arithmetic operators *) }
| expr MINUS  expr                { Binop($1, Sub, $3) }
| expr TIMES  expr                { Binop($1, Mul, $3) }
| expr DIVIDE expr                { Binop($1, Div, $3) }
| expr LAYER  expr                { Binop($1, Layer, $3)  (* Layer operator *) }
| expr REPEAT expr                { Binop($1, Rep, $3)    (* Array init operator *) }
| expr ARRCAT expr                { Binop($1, Arrcat, $3) (* Array concat operator *) }
| expr EQ     expr                { Binop($1, Eq, $3)     (* Equality operators *) }
| expr NEQ    expr                { Binop($1, Neq, $3) }
| expr GT     expr                { Binop($1, Gt, $3) }
| expr GTE    expr                { Binop($1, Gte, $3) }
| expr LT     expr                { Binop($1, Lt, $3) }
| expr LTE    expr                { Binop($1, Lte, $3) }
| ID LSBRACE expr RSBRACE ASSIGN expr
                                  { Arrmod($1, $3, $6)    (* modify array *) }
| ID LSBRACE expr RSBRACE         { Arrget($1, $3)        (* array reference *) }
| literal                         { Lit($1)               (* Lit($1) Any literal. Should make  
                                                             type-checking easier *) }
| ID                              { Id($1)                (* Identifier *) }


actuals_opt:
                { [] }
| actuals_list   { List.rev $1 }

actuals_list:
  expr                    { [$1] }
| actuals_list COMMA expr { $3 :: $1 }


literal:
  INT_LIT                          { Intlit($1) }
| PITCH_LETTER INT_LIT             { Pitchlit($1, $2) }
| PITCH_LETTER                     { Pitchlit($1, 4) }
| LPAREN expr COMMA expr RPAREN    { Tuple($2, $4) }
