(* 
Semantic AST 
by Andrew OReilly, Stephanie Huang
*)

open Ast

exception Type_mismatch of string
exception Missing_variable of string
exception Missing_function of string
exception Invalid_type of string

module StringMap = Map.Make(String) ;;

(* Types for semantic checking *)
type s_type = SInt | SPitch | SDur | SNote | SChord | SPhrase 
  | SScore | SArray | None

(* cmajor types *)
type c_type =
    Int of int
  | Pitch of int * int
  | Dur of int * int
  | Note of c_type * c_type
  | Chord of c_type array * c_type
  | Phrase of c_type array
  | Score of c_type array
  | Array of s_type * c_type array
  | None

(* Get the semantic type of a storage type *)
let rec c_to_s_type = function
    Int x -> SInt
  | Pitch (l,o) -> SPitch
  | Dur (a,b) -> SDur
  | Note (p,d) -> SNote
  | Chord (p,d) -> SChord
  | Phrase x -> SPhrase
  | Score x -> SScore
  | Array (s,l) -> SArray
  | None -> None

(* Get string representation of a c_type *)
let rec c_type_str = function
    Int(x) -> "Int(" ^ string_of_int x ^ ")"
  | Pitch(l, o) -> "Pitch("^(string_of_int l)^","^(string_of_int o)^")"
  | Dur(a,b) -> "Dur("^(string_of_int a)^","^(string_of_int b)^")"
  | Note(p,d) -> "Note("^(c_type_str p)^","^(c_type_str d)^")"
  | Chord(p,d) -> "Chord("^(c_type_str (Array(SPitch, p)))^","^(c_type_str d)^")"
  | Phrase(p) -> "Phrase("^(c_type_str (Array(SChord, p)))^")"
  | Score(s) -> "Score("^(c_type_str (Array(SPhrase, s)))^")"
  | Array(s,a) -> "Array("^Array.fold_left (
      fun prev x -> 
        if (prev = "") then (c_type_str x) else prev^","^(c_type_str x)
    ) "" a^")"
  | None -> "None"

