{ 
(*
Scanner for CMajor
By PLT Sandwich
Andrew OReilly, Stephanie Huang
*)
open Parser 
}

(* let pitch = '$' [ 'A'-'G' ] [ '#' 'b' ]? <- Not used, but handy *)
let id = [ '_' 'A'-'Z' 'a'-'z' ] [ '_' 'A'-'Z' 'a'-'z' '0'-'9' ]*
let typename = "int" | "pitch" | "dur" | "note" | "chord" | "phrase" | "score"

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf }    (* Whitespace *)
| "/*"                 { bcomment lexbuf } (* Block comments (/* */) *)
| "//"                 { icomment lexbuf } (* Inline comments (//) *)

| '('                     { LPAREN }       (* Punctuation *)
| ')'                     { RPAREN }
| '{'                     { LCBRACE }
| '}'                     { RCBRACE }
| '['                     { LSBRACE }
| ']'                     { RSBRACE }

| ';'                     { SEMI }
| ','                     { COMMA }

| '='                     { ASSIGN }
| '+'                     { PLUS }
| '-'                     { MINUS }
| '*'                     { TIMES }
| '/'                     { DIVIDE }

| "=="                    { EQ }
| "!="                    { NEQ }
| '>'                     { GT }
| ">="                    { GTE }
| '<'                     { LT }
| "<="                    { LTE }

| '^'                     { LAYER }
| "**"                    { REPEAT }
| "++"                    { ARRCAT }

(* keywords *)
| "if"                    { IF }
| "else"                  { ELSE }
| "return"                { RETURN }
| "for"                   { FOR }


(* Pitch literals are split into separate symbols to make parsing easier *)
| '$'                     { ptoken lexbuf }
| ['0'-'9']+ as lit       { INT_LIT(int_of_string lit) } (* integers *)
| typename as type_str    { TYPENAME(type_str) }         (* type names *)
| id as id_str            { ID(id_str) }                 (* identifiers *)
| eof                     { EOF }                        (* end-of-file *)

and bcomment = parse
  "*/" { token lexbuf }
  | _    { bcomment lexbuf }
  
and icomment = parse
  '\n'   { token lexbuf } (* Should this be quoted? *)
  | _    { icomment lexbuf }

and ptoken = parse
  | "A"                    { PITCH_LETTER(0) }
  | "A#" | "Bb"            { PITCH_LETTER(1) }
  | "B"  | "Cb"            { PITCH_LETTER(2) }
  | "B#" | "C"             { PITCH_LETTER(3) }
  | "C#" | "Db"            { PITCH_LETTER(4) }
  | "D"                    { PITCH_LETTER(5) }
  | "D#" | "Eb"            { PITCH_LETTER(6) }
  | "E"  | "Fb"            { PITCH_LETTER(7) }
  | "E#" | "F"             { PITCH_LETTER(8) }
  | "F#" | "Gb"            { PITCH_LETTER(9) }
  | "G"                    { PITCH_LETTER(10) }
  | "G#" | "Ab"            { PITCH_LETTER(11) }
  | 'R'                    { PITCH_LETTER(-1) }
  | _ { token lexbuf }