#!/bin/bash

# Set time limit for all operations
ulimit -t 30

globallog=testall.log
rm -f #globallog
error=0
globalerror=0
numtests=0
numpass=0
keep=0

printf "==============================\n"
printf "           Testing            \n"
printf "==============================\n"

Usage() {
    echo "Usage: test.sh [options] [.cmaj files]"
    echo "   -k keep intermediate files"
    echo "   -h help"
    exit 1
}

SignalError() {
    if [ $error -eq 0 ] ; then
        echo "FAILED"
        error=1
    fi
        echo "  $1"
}

# Compare <outfile> <reffile> <difffile>
# Compares the outfile with reffile.  Differences, if any, written to difffile
Compare() {
    generatedfiles="$generatedfiles $3"
    echo diff -b $1 $2 ">" $3 1>&2
    diff -b "$1" "$2" > "$3" 2>&1 || {
        SignalError "$1 differs"
        echo "FAILED $1 differs from $2" 1>&2
    }
}

# Run <args>
# Report the command, run it, and report any errors
Run() {
    echo $* 1>&2
    eval $* || {
        SignalError "$1 failed on $*"
        return 1
    }
}

Check() {
    error=0
    numtests=$((numtests+1))

    basename=`echo $1 | sed 's/.*\\///
                            s/.cmaj//'`
    reffile=`echo $1 | sed 's/.cmaj$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo 1>&2
    echo ">>> Testing $basename:" 1>&2

    generatedfiles=""

    generatedfiles="$generatedfiles ${basename}.i.out" &&
    echo "${basename}.cmaj" >> output.txt &&
    Run "./cmajor < tests/${basename}.cmaj" >> ${basename}.i.out &&
    cat ${basename}.i.out >> output.txt &&
    printf "\n" >> output.txt &&
    Compare ${basename}.i.out ${reffile}.out ${basename}.diff
    if test -f "out.csv"; then
        Compare out.csv ${reffile}.csv ${basename}-csv.diff
    fi
    if test -f "out.csv"; then
        rm out.csv
    fi


    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
        if [ $keep -eq 0 ] ; then
            rm -f $generatedfiles
        fi
        numpass=$((numpass+1))
        echo "${basename}: PASS" 1>&2
    else
    if [ $delete -eq 1 ] ; then
        rm -f $generatedfiles
    fi
        echo "${basename}: FAIL" 1>&2
        echo "${basename}.cmaj: FAIL" >> failed.txt
        printf "\n" >> failed.txt
        printf "Failure message:" >> failed.txt
        printf "\n" >> failed.txt
        cat $generatedfiles >> failed.txt
        printf "==============================\n" >> failed.txt
        rm -f $generatedfiles
        globalerror=$error
    fi

}

while getopts kdpsh c; do
    case $c in
        k) # Keep intermediate files
            keep=1
            ;;
        h) # Help
            Usage
            ;;
    esac
done

shift `expr $OPTIND - 1`

files="tests/*.cmaj"

for file in $files
    do
        case $file in
        *)
            Check $file
            ;;
    esac
done

printf "\n"
printf "==============================\n"
printf "           Results            \n"
echo "Number of tests ran: $numtests" 1>&2
echo "Number of tests pass: $numpass" 1>&2
printf "==============================\n"

exit $globalerror
